@@ -339,7 +339,18 @@ nsDOMWorkerXHR::nsDOMWorkerXHR(nsDOMWorker* aWorker)
 nsDOMWorkerXHR::~nsDOMWorkerXHR()
 {
   if (mXHRProxy) {
-    mXHRProxy->Destroy();
+    if (!NS_IsMainThread()) {
+      nsCOMPtr<nsIRunnable> runnable =
+        NS_NEW_RUNNABLE_METHOD(nsDOMWorkerXHRProxy, mXHRProxy.get(), Destroy);
+
+      if (runnable) {
+        mXHRProxy = nsnull;
+        NS_DispatchToMainThread(runnable, NS_DISPATCH_NORMAL);
+      }
+    }
+    else {
+      mXHRProxy->Destroy();
+    }
   }
 }
 
@@ -688,6 +699,12 @@ nsDOMWorkerXHR::Send(nsIVariant* aBody)
     return NS_ERROR_ABORT;
   }
 
+  if (mWorker->IsClosing() && !mXHRProxy->mSyncRequest) {
+    // Cheat and don't start this request since we know we'll never be able to
+    // use the data.
+    return NS_OK;
+  }
+
   nsresult rv = mXHRProxy->Send(aBody);
   NS_ENSURE_SUCCESS(rv, rv);
 
@@ -703,6 +720,12 @@ nsDOMWorkerXHR::SendAsBinary(const nsAString& aBody)
     return NS_ERROR_ABORT;
   }
 
+  if (mWorker->IsClosing() && !mXHRProxy->mSyncRequest) {
+    // Cheat and don't start this request since we know we'll never be able to
+    // use the data.
+    return NS_OK;
+  }
+
   nsresult rv = mXHRProxy->SendAsBinary(aBody);
   NS_ENSURE_SUCCESS(rv, rv);
 
