@@ -42,7 +42,6 @@
 #include "nsISupports.h"
 #include "nsCoord.h"
 #include "nsRect.h"
-#include "nsIWidget.h"
 #include "nsIRenderingContext.h"
 // XXX we need only gfxTypes.h, but we cannot include it directly.
 #include "gfxPoint.h"
@@ -169,21 +168,10 @@ const PRUint8 kUseAltDCFor_CREATERC_PAINT  = 0x04; // Use when creating Renderin
 const PRUint8 kUseAltDCFor_SURFACE_DIM     = 0x08; // Use it for getting the Surface Dimensions
 #endif
 
-// eca27eb2-ef6f-4142-bf77-89c14e595171
+// 40aebd88-a82b-48b0-8a44-be51510064a7
 #define NS_IDEVICE_CONTEXT_IID   \
-{ 0xeca27eb2, 0xef6f, 0x4142, \
-  { 0xbf, 0x77, 0x89, 0xc1, 0x4e, 0x59, 0x51, 0x71 } }
-
-//a cross platform way of specifying a native palette handle
-typedef void * nsPalette;
-
-  //structure used to return information about a device's palette capabilities
-  struct nsPaletteInfo {
-     PRPackedBool  isPaletteDevice;
-     PRUint16      sizePalette;  // number of entries in the palette
-     PRUint16      numReserved;  // number of reserved palette entries
-     nsPalette     palette;      // native palette handle
-  };
+{ 0x40aebd88, 0xa82b, 0x48b0, \
+  { 0x8a, 0x44, 0xbe, 0x51, 0x51, 0x00, 0x64, 0xa7 } }
 
   typedef enum {
     eSystemFont_Caption,         // css2
@@ -215,10 +203,10 @@ public:
 
   /**
    * Initialize the device context from a widget
-   * @param aWidget a native widget to initialize the device context from
+   * @param aWidget a widget to initialize the device context from
    * @return error status
    */
-  NS_IMETHOD  Init(nsNativeWidget aWidget) = 0;
+  NS_IMETHOD  Init(nsIWidget* aWidget) = 0;
 
   /**
    * Initialize the device context from a device context spec
@@ -378,11 +366,6 @@ public:
    */
   NS_IMETHOD GetDepth(PRUint32& aDepth) = 0;
 
-  /**
-   * Returns information about the device's palette capabilities.
-   */
-  NS_IMETHOD GetPaletteInfo(nsPaletteInfo& aPaletteInfo) = 0;
-
   /**
    * Get the size of the displayable area of the output device
    * in app units.
