@@ -25,7 +25,6 @@
  *   Igor Bukanov
  *   Ethan Hugg
  *   Milen Nankov
- *   Jeff Walden <jwalden+code@mit.edu>
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -41,98 +40,29 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '13.4.4.26.js';
+gTestfile = '13.4.4.6.js';
 
-START("13.4.4.26 - XML normalize()");
+START("13.4.4.6 - XML child()");
 
-TEST(1, true, XML.prototype.hasOwnProperty("normalize"));
+TEST(1, true, XML.prototype.hasOwnProperty("child"));
 
-XML.ignoreWhitespace = false;
-XML.prettyPrinting = false;
+emps =
+<employees>
+    <employee id="0"><name>Jim</name><age>25</age></employee>
+    <employee id="1"><name>Joe</name><age>20</age></employee>
+</employees>;
 
-var x = <alpha> <bravo> one </bravo> </alpha>;
-
-TEST_XML(2, "<alpha> <bravo> one </bravo> </alpha>", x);
-x.normalize();
-TEST_XML(3, "<alpha> <bravo> one </bravo> </alpha>", x);
+correct =
+<employee id="0"><name>Jim</name><age>25</age></employee> +
+<employee id="1"><name>Joe</name><age>20</age></employee>;
 
 
-// First, test text node coalescing
+TEST(2, correct, emps.child("employee"));
 
-delete x.bravo[0];
+TEST(3, <name>Joe</name>, emps.employee[1].child("name"));
 
-TEST_XML(4, "<alpha>  </alpha>", x);
-TEST(5, 2, x.children().length());
+correct = <employee id="1"><name>Joe</name><age>20</age></employee>;
 
-x.normalize();
-
-TEST_XML(6, "<alpha>  </alpha>", x);
-TEST(7, 1, x.children().length());
-
-// check that nodes are inserted in the right place after a normalization
-
-x.appendChild(<bravo> fun </bravo>);
-
-TEST_XML(8, "<alpha>  <bravo> fun </bravo></alpha>", x);
-TEST(9, 2, x.children().length());
-
-// recursive nature
-
-var y = <charlie> <delta/> </charlie>;
-
-TEST(10, 3, y.children().length());
-
-x.appendChild(y);
-delete y.delta[0];
-
-TEST(11, 2, y.children().length());
-
-x.normalize();
-
-TEST(12, 1, y.children().length());
-TEST(13, 1, x.charlie.children().length());
-
-
-
-// Second, test empty text node removal
-
-x = <alpha><beta/></alpha>;
-
-TEST_XML(14, "<alpha><beta/></alpha>", x);
-TEST(15, 1, x.children().length());
-
-x.appendChild(XML());
-
-TEST_XML(16, "<alpha><beta/></alpha>", x);
-TEST(17, 2, x.children().length());
-
-x.normalize();
-
-TEST_XML(18, "<alpha><beta/></alpha>", x);
-TEST(19, 1, x.children().length());
-
-x.appendChild(XML(" "));
-
-TEST_XML(20, "<alpha><beta/> </alpha>", x);
-TEST(21, 2, x.children().length());
-
-x.normalize();
-
-// normalize does not remove whitespace-only text nodes
-TEST_XML(22, "<alpha><beta/> </alpha>", x);
-TEST(23, 2, x.children().length());
-
-y = <foo/>;
-y.appendChild(XML());
-
-TEST(24, 1, y.children().length());
-
-x.appendChild(y);
-
-// check recursive nature
-
-x.normalize();
-
-TEST(25, 0, y.children().length());
+TEST(4, correct, emps.child(1));
 
 END();
