@@ -40,37 +40,35 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '13.5.4.11.js';
+gTestfile = '13.5.4.3.js';
 
-START("13.5.4.11 - XMLList hasOwnProperty()");
+START("13.5.4.3 - XMLList attributes()");
 
-TEST(1, true, XMLList.prototype.hasOwnProperty("hasOwnProperty"));
- 
+TEST(1, true, XMLList.prototype.hasOwnProperty("attributes"));
 
-x =
-<>
-<alpha attr1="value1">
-    <bravo>one</bravo>
-    <charlie>
-        two
-        three
-        <echo>four</echo>
-    </charlie>
-    <delta />
-</alpha>
-<delta>
-    <echo>five</echo>
-</delta>
-</>;
+// Test with XMLList of size 0
+x = new XMLList()
+TEST(2, "xml", typeof(x.attributes()));
+TEST_XML(3, "", x.attributes());
 
-// Returns true for elements/attributes
-TEST(2, true, x.hasOwnProperty("bravo"));
-TEST(3, true, x.hasOwnProperty("@attr1"));
-TEST(4, false, x.hasOwnProperty("foobar"));
-TEST(5, true, x.hasOwnProperty("echo"));
+// Test with XMLList of size 1
+x += <alpha attr1="value1" attr2="value2">one</alpha>;
+
+TEST(4, "xml", typeof(x.attributes()));
+correct = new XMLList();
+correct += new XML("value1");
+correct += new XML("value2");
+TEST(5, correct, x.attributes());
+
+// Test with XMLList of size > 1
+x += <bravo attr3="value3" attr4="value4">two</bravo>;
+
+TEST(6, "xml", typeof(x.attributes()));
+correct = new XMLList();
+correct += new XML("value1");
+correct += new XML("value2");
+correct += new XML("value3");
+correct += new XML("value4");
+TEST(7, correct, x.attributes());
 
-// Test for XML Prototype Object - returns true for XML methods.
-TEST(5, true, XMLList.prototype.hasOwnProperty("toString"));
-TEST(6, false, XMLList.prototype.hasOwnProperty("foobar"));
- 
 END();
\ No newline at end of file
