@@ -36,10 +36,10 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '7.4.2-15-n.js';
+gTestfile = '7.4.2-6-n.js';
 
 /**
-   File Name:          7.4.2-15-n.js
+   File Name:          7.4.2-6.js
    ECMA Section:       7.4.2
 
    Description:
@@ -62,16 +62,16 @@ gTestfile = '7.4.2-15-n.js';
    Date:               12 november 1997
 
 */
-var SECTION = "7.4.1-15-n";
+var SECTION = "7.4.2-6-n";
 var VERSION = "ECMA_1";
 startTest();
 var TITLE   = "Keywords";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-DESCRIPTION = "var typeof = true";
+DESCRIPTION = "var function = true";
 EXPECTED = "error";
 
-new TestCase( SECTION,  "var typeof = true",     "error",    eval("var typeof = true") );
+new TestCase( SECTION,  "var function = true",     "error",    eval("var function = true") );
 
 test();
