@@ -15,12 +15,12 @@
  * The Original Code is JavaScript Engine testing utilities.
  *
  * The Initial Developer of the Original Code is
- * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2005
+ * Netscape Communications Corp.
+ * Portions created by the Initial Developer are Copyright (C) 2002
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Igor Bukanov <igor@mir2.org>
- *                 Bob Clary <bob@bclary.com>
+ * Contributor(s):
+ *   pschwartau@netscape.com
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,46 +36,39 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-230216-1.js';
+/*
+ *
+ * Date:    16 Mar 2002
+ * SUMMARY: Shouldn't crash if define |var arguments| inside a function
+ * See http://bugzilla.mozilla.org/show_bug.cgi?id=131510
+ *
+ */
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 230216;
-var summary = 'check for numerical overflow in regexps in back reference and bounds for {} quantifier';
-var actual = '';
-var expect = '';
-var status = '';
-
+var gTestfile = 'regress-131510-001.js';
+var BUGNUMBER = 131510;
+var summary = "Shouldn't crash if define |var arguments| inside a function";
 printBugNumber(BUGNUMBER);
-printStatus (summary);
+printStatus(summary);
 
-status = inSection(1) + ' check for overflow in backref';
 
-actual = 'undefined'; 
-expect = false;
+function f() {var arguments;}
+f();
 
-try
-{
-  actual = eval('/(a)\21474836481/.test("aa")');
-}
-catch(e)
-{
-  status += ' Error: ' + e;
-}
 
-reportCompare(expect, actual, status);
+/*
+ * Put same example in function scope instead of global scope
+ */
+function g() { function f() {var arguments;}; f();};
+g();
 
-status = inSection(2) + ' check for overflow in backref';
 
-actual = 'undefined'; 
-expect = false;
+/*
+ * Put these examples in eval scope
+ */
+var s = 'function f() {var arguments;}; f();';
+eval(s);
 
-try
-{ 
-  actual = eval('/a\21474836480/.test("")');
-}
-catch(e)
-{
-  status += ' Error: ' + e;
-}
-
-reportCompare(expect, actual, status);
+s = 'function g() { function f() {var arguments;}; f();}; g();';
+eval(s);
 
+reportCompare('No Crash', 'No Crash', '');
