@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2006
+ * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Jesse Ruderman
+ * Contributor(s): Gary Kwong
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,84 +35,32 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-351503-01.js';
+var gTestfile = 'regress-452498-168-1.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 351503;
-var summary = 'decompilation of TypeError messages';
+var BUGNUMBER = 452498;
+var summary = 'TM: upvar2 regression tests';
 var actual = '';
 var expect = '';
 
-printBugNumber(BUGNUMBER);
-printStatus (summary);
+//-------  Comment #168  From  Gary Kwong
 
-expect = 'TypeError: can\'t convert Object to string';
-try
-{
-  function p() { return { toString: null } } "fafa".replace(/a/g, p);
-  actual = 'No Error';
-}
-catch(ex)
-{
-  actual = ex + '';
-}
-reportCompare(expect, actual, summary + ': 1');
+//-----------------------------------------------------------------------------
+test();
+//-----------------------------------------------------------------------------
 
-expect = 'TypeError: can\'t convert Object to string';
-try
+function test()
 {
-  var a=1; b=2; c={toString: null}; "hahbhc".replace(/[abc]/g, eval);
-  actual = 'No Error';
-}
-catch(ex)
-{
-  actual = ex + '';
-}
-reportCompare(expect, actual, summary + ': 2');
+  enterFunc ('test');
+  printBugNumber(BUGNUMBER);
+  printStatus (summary);
 
-expect = 'TypeError: can\'t convert ({toString:(function () {yield 4;})}) to primitive type';
-try
-{
-  3 + ({toString:function(){yield 4} });
-  actual = 'No Error';
-}
-catch(ex)
-{
-  actual = ex + '';
-}
-reportCompare(expect, actual, summary + ': 3');
+// Assertion failure: (fun->u.i.script)->upvarsOffset != 0, at ../jsfun.cpp:1543
 
-expect = 'TypeError: ({}) is not a function';
-try
-{
-  3 + ({toString:({}) }) ;
-  actual = 'No Error';
-}
-catch(ex)
-{
-  actual = ex + '';
-}
-reportCompare(expect, actual, summary + ': 4');
+  (
+    new Function("const x = (function () { if (1e+81){} else{x} } )"
+      ))();
 
-expect = 'TypeError: 3 is not a function';
-try
-{
-  ([5].map)(3);
-  actual = 'No Error';
-}
-catch(ex)
-{
-  actual = ex + '';
-}
-reportCompare(expect, actual, summary + ': 5');
+  reportCompare(expect, actual, summary);
 
-expect = 'TypeError: 5 is not a function';
-try
-{
-  (1 + (4))();
-  actual = 'No Error';
+  exitFunc ('test');
 }
-catch(ex)
-{
-  actual = ex + '';
-}
-reportCompare(expect, actual, summary + ': 6');
