@@ -252,6 +252,16 @@ oggz_read_free_pbuffer_entry(OggzBufferedPacket *p) {
 
 }
 
+OggzDListIterResponse
+oggz_read_free_pbuffers(void *elem)
+{
+  OggzBufferedPacket *p = (OggzBufferedPacket *)elem;
+
+  oggz_read_free_pbuffer_entry(p);
+
+  return DLIST_ITER_CONTINUE;
+}
+
 OggzDListIterResponse
 oggz_read_update_gp(void *elem) {
 
@@ -497,9 +507,7 @@ oggz_read_sync (OGGZ * oggz)
     }
 
     /* If we've got a stop already, don't read more data in */
-    if (cb_ret == OGGZ_STOP_OK || 
-	cb_ret == OGGZ_STOP_ERR || 
-	cb_ret == OGGZ_ERR_HOLE_IN_DATA) 
+    if (cb_ret < 0 || cb_ret == OGGZ_STOP_OK) 
       return cb_ret;
 
     if(oggz_read_get_next_page (oggz, &og) < 0)
